#include <bitset>
#include <SDL.h>
#include "../Camera/Camera.h"
#include "../Arch/Id.h"
#include "../Arch/Singleton.h"

#include "../Components/FwdDecs.h"

namespace Input_
{
enum InputBit
{
	INVALID_STATE,
	RUN,
	JUMP,
	BACKPEDDLE,
	TURN_LEFT,
	TURN_RIGHT,
	TURN_UP,
	TURN_DOWN,
	STRAFE_LEFT,
	STRAFE_RIGHT,
	STRAFE_UP,
	STRAFE_DOWN,
	WORLD_ROTATE,
	WORLD_ROTATE_PREVIEW,
	OBJECT_TRANSLATE,
	OBJECT_ROTATE,
	SHIFT_HOLD,
	CONTROL_HOLD,
	CONSOLE_TOGGLE,
	QUIT,
	DEBUG_CONTROL,
	ANGLE_SNAP,
	TRANS_Z_F,
	TRANS_Z_B,
	TRANS_X_L,
	TRANS_X_R,
	TRANS_Y_U,
	TRANS_Y_D,
	ROT_X_F,
	ROT_X_B,
	ROT_Y_L,
	ROT_Y_R,
	ROT_Z_L,
	ROT_Z_R,
	TOTAL_BITS
};
enum WRInputBit//@TEST
{
	WORLD_INVALID_STATE,
	WORLD_TURN_LEFT,  
	WORLD_TURN_RIGHT, 
	WORLD_TURN_UP,
	WORLD_TURN_DOWN,
	WORLD_TOTAL_BITS
};

typedef std::bitset<TOTAL_BITS> InputBits;
typedef std::bitset<WORLD_TOTAL_BITS> WRInputBits;

class InputHandler : public Singleton<InputHandler>
{
	public:
	  ~InputHandler();

	  void setKeyState(SDLKey key, bool state);
	  void setState(InputBit input, bool state);
	  void setWRState(WRInputBit WRinput, bool state);
	  void setRotationAngles(const Math_::Vector2& rvec);

	  void translateObject(const Math_::Vector3& disp) const;
	  void rotateObject(float phi, float theta, float rho) const;
	  bool clearStates(void);
	  void update(Camera_::Camera& cam) const;
	  bool worldRotateState(void);
	  bool worldPreviewRotateState(void);
	  void previewRotateAction(void);
	  bool debugControlState(void);
	  bool angleSnapState(void);
	  bool objectTranslatieState(void);
	  bool objectRotateState(void);
	  bool shiftState(void);
	  bool controlState(void);


	  void setFastKeyRepeat(bool b);

	  void setSelectionFreeze(bool freeze);
	  void setSelectionVelocity( const Math_::Vector2 &vel );

	  const Math_::Vector2 &getMotionVelocity() const;
	  void setMotionVelocity( const Math_::Vector2 &vel );

	  void rectifyRotationAngle(void);

	  bool isMouseLocked() const;
	  void setMouseLock( bool lock );

	  const Id_::ObjId_t getSelectedObject(void) const;
	  void setSelectedObject(Id_::ObjId_t objectID);

	  void setJohnny(Component_::Entity* johnny);
	  const Component_::Entity* getjohnny(void);

	  const Math_::Vector3 &getJohnnyMovement() const;

	private:
	  InputHandler();
	  friend class Singleton<InputHandler>;

	  mutable Math_::Vector2 rotationAngles;
	  
	  // FIXME - johnny movement yuck
	  mutable Math_::Vector3 johnnyMove;

	  Math_::Vector2 motionVelocity;

	  // FIXME - for locking mouse right now
	  bool lockMouse;

	  InputBits inputState;
	  WRInputBits WRinputState;
	  Id_::ObjId_t selectedObject;
	  Component_::Entity* johnny;
};

void inputUpdate(SDL_Event* input, InputHandler& handler);
}
//END OF Input_
